(*
**  GEcho 1.02+ Developers Kit Demonstration Source Code
**
**  Written by Gerard J. van der Land
**
**  Copyright (C) 1992-1994 Gerard J. van der Land. All rights reserved.
**
**  Last updates: 09-Feb-94
*)

{$I gestruct.inc}
{$L pascal.obj}

var
   sys          : SETUP_GE;
   AreaHdr      : AREAFILE_HDR;
   Area         : AREAFILE_GE;
   AreaB, AreaP : ^AREAFILE_GEX;
   Connections  : array [1..MAXCONNECTIONS] of CONNECTION;

   SetupGE, AreaFileGE, AreaFileGEX   : file;
   areas, arearecsize, connectionsize : word;
   counter, result, records, node     : word;
   buffer : string[255];
   temp   : longint;

procedure c2p(dest : string ; src : pointer; max : byte); external;
procedure p2c(dest : pointer; src : string ; max : byte); external;

begin

(*
**  Opening and reading SETUP.GE
*)

{$I-}
   assign(SetupGE, 'SETUP.GE');
   reset(SetupGE, 1);
{$I+}
   if IOresult <> 0 then
   begin
      writeln('Unable to open SETUP.GE');
      halt(255)
   end;
   blockread(SetupGE, sys, sizeof(SETUP_GE), result);
   close(SetupGE);

   if result < sizeof(SETUP_GE) then
   begin
      writeln('Error reading SETUP.GE');
      halt(255)
   end;

   if (sys.sysrev <> GE_THISREV) then
   begin
      writeln('System file revision level mismatch');
      halt(251)
   end;

   c2p(buffer, @sys.username[1], 35);  (* C -> Pascal *)
   writeln('SysOp: ', buffer);

   p2c(@sys.username[1], 'gerard van.der.land', 35);  (* Pascal -> C *)

(*
**  Opening AREAFILE.GE and checking the header
*)

{$I-}
   assign(AreaFileGE, 'AREAFILE.GE');
   reset(AreaFileGE, 1);
{$I+}
   if IOresult <> 0 then
   begin
      writeln('Unable to open AREAFILE.GE');
      halt(255)
   end;

   blockread(AreaFileGE, AreaHdr, sizeof(AREAFILE_HDR), result);
   if result < sizeof(AREAFILE_HDR) then
   begin
      writeln('Error reading AREAFILE header');
      halt(255)
   end;

   if (AreaHdr.hdrsize < sizeof(AREAFILE_HDR)) OR
      (AreaHdr.recsize < sizeof(AREAFILE_GE)) then
   begin
      writeln('Incompatible AREAFILE record size');
      halt(251)
   end;

   connectionsize := AreaHdr.maxconnections * sizeof(CONNECTION);
   arearecsize := AreaHdr.recsize + connectionsize;

(*
**  Reading AREAFILE.GE using AREAFILE.GEX index file
*)

{$I-}
   assign(AreaFileGEX, 'AREAFILE.GEX');
   reset(AreaFileGEX, 1);
{$I+}
   if IOresult <> 0 then
   begin
      writeln('Unable to open AREAFILE.GEX');
      halt(255)
   end;

   temp := filesize(AreaFileGEX) div sizeof(AREAFILE_GEX);
   if temp > MAXAREAS then
   begin
      writeln('Index file too large');
      close(AreaFileGEX);
      halt(255)
   end;
   areas := word(temp);

   if maxavail < areas * sizeof(AREAFILE_GEX) then
   begin
      writeln('Insufficient memory');
      close(AreaFileGEX);
      halt(254)
   end;

   getmem(AreaB, areas * sizeof(AREAFILE_GEX));

   blockread(AreaFileGEX, AreaB^, areas * sizeof(AREAFILE_GEX));
   close(AreaFileGEX);

   AreaP := AreaB;
   for counter := 1 to areas do
   begin
      seek(AreaFileGE, AreaP^.offset);
      blockread(AreaFileGE, Area, sizeof(AREAFILE_GE), result);
      if result = sizeof(AREAFILE_GE) then
      begin
         c2p(buffer, @Area.name, 50);
         writeln(counter:4, '  ' , buffer);

	 (* Print list of nodes connected to this area *)
	 seek(AreaFileGE, AreaP^.offset + AreaHdr.recsize);
	 blockread(AreaFileGE, Connections, connectionsize, result);

         if (result = connectionsize) then
         begin
            node := 1;
            while (node <= AreaHdr.maxconnections) AND (Connections[node].address.zone <> 0) do
            begin
	       writeln('      ', node:3, '  ',
                  Connections[node].address.zone, ':',
                  Connections[node].address.net,  '/',
                  Connections[node].address.node, '.',
                  Connections[node].address.point);
               inc(node)
	    end
         end
      end;
      inc(AreaP)
   end;

   freemem(AreaB, areas * sizeof(AREAFILE_GEX));

(*
**  Reading AREAFILE.GE sequentially
*)

   records := (filesize(AreaFileGE) - AreaHdr.hdrsize) div arearecsize;

   areas := 0;
   for counter := 0 to records-1 do
   begin
      seek(AreaFileGE, AreaHdr.hdrsize + longint(arearecsize) * longint(counter));
      blockread(AreaFileGE, Area, sizeof(AREAFILE_GE), result);
      if result = sizeof(AREAFILE_GE) then
      begin
         if (Area.options and REMOVED) = 0 then
         begin
            inc(areas);
            c2p(buffer, @Area.name, 50);
            writeln(areas:4, '  ' , buffer)
         end
      end
   end;
   close(AreaFileGE)

end.

(* end of file "ge_demo.pas" *)
